rem Modify [] according to your environment.

set HOMEPATH=[PATH]
rem start mlterm.exe -S sample -e plink.exe [-telnet/-ssh/-rlogin/-raw] [HOST]
rem start mlterm.exe -S sample --serv [<proto>://<user>@<host>:<port>:<encoding>]
start mlterm.exe -S sample
exit
