/*
 *	$Id$
 */

#ifndef  __MC_LETTER_SPACE_H__
#define  __MC_LETTER_SPACE_H__


#include  <gtk/gtk.h>


GtkWidget *  mc_letter_space_config_widget_new(void) ;

void  mc_update_letter_space(void) ;


#endif
